@@ -85,6 +85,8 @@ class LogEntry(Schema):
 class LogSummaryReport(Schema):
     N = fields.Int()
 
+class Summary(Schema):
+    pass
 
 class QueueLogEntry(Schema):
     state = fields.Str()
@@ -105,6 +107,44 @@ class DataFact(Schema):
 
 ## === views
 
+class SummaryView(SwaggerView):
+    operationId = "summary"
+    parameters = [
+        {
+            "name": "queue",
+            "in": "query",
+            "type": "string",
+            "required": False,
+        },
+    ]
+    responses = {
+        200: {
+            "description": "A summary of tasks",
+            "schema": Summary,
+        }
+    }
+
+    def get(self):
+        """
+        get summary of tasks
+        """
+
+        queue = dqueue.core.Queue(request.args.get('queue', 'default'))
+
+        tasks = queue.summary
+
+        logger.info("got summary: %s", tasks)
+
+        return jsonify(
+                tasks=tasks
+            )
+
+app.add_url_rule(
+         '/tasks/summary',
+          view_func=SummaryView.as_view('summary_tasks'),
+          methods=['GET']
+)
+
 class TaskListView(SwaggerView):
     operationId = "listTasks"
     parameters = [
@@ -118,10 +158,10 @@ class TaskListView(SwaggerView):
             "name": "state",
             "in": "query",
             "type": "string",
-            "enum": ["submitted", "waiting", "done", "all"],
+            "enum": ["submitted", "waiting", "done", "any", "failed", "running"],
             "required": False,
-            "default": "all",
-        }
+            "default": "any",
+        },
     ]
     responses = {
         200: {
@@ -136,8 +176,9 @@ class TaskListView(SwaggerView):
         """
 
         queue = dqueue.core.Queue(request.args.get('queue', 'default'))
+        state = request.args.get('state', 'any')
 
-        tasks = [e for e in tools.list_tasks(include_task_data=True)]
+        tasks = [e for e in tools.list_tasks(include_task_data=True, state=state)]
 
         #tasks = queue.list_tasks(decode=True) #state=state)
 
@@ -169,6 +210,12 @@ class WorkerOffer(SwaggerView):
                     'required': False,
                     'type': 'string',
                 },
+                {
+                    'name': 'update_expected_in_s',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'number',
+                },
             ]
 
     responses = {
@@ -182,10 +229,11 @@ class WorkerOffer(SwaggerView):
         }
 
     def get(self, worker_id):
+        update_expected_in_s = request.args.get('update_expected_in_s', -1, type=float)
         queue = dqueue.core.Queue(request.args.get('queue', 'default'), worker_id=worker_id)
 
         try:
-            task = queue.get()
+            task = queue.get(update_expected_in_s)
             logger.warning("got task: %s", task)
             return jsonify(
                     task.as_dict,
@@ -461,7 +509,11 @@ class WorkerDataConsultFact(SwaggerView):
                 try:
                     meta, payload  = odakb.datalake.restore(dag_bucket, return_metadata=True)
                 except minio.error.NoSuchBucket:
-                    raise Exception("bucket was just suppose to exists!")
+                    logger.error("bucket was just supposed to exists! race condition?")
+                    return Response(
+                              f"no such dag! bucket: {dag_bucket}",
+                              status=400,
+                           )
 
                 assert payload['dag'] == dag
                 
@@ -579,6 +631,36 @@ app.add_url_rule(
           methods=['GET']
 )
 
+class ExpireTasks(SwaggerView):
+    operationId = "expire"
+
+    # locally or remotely?
+
+    parameters = [
+            ]
+
+    responses = {
+            200: {
+                    'description': 'expired',
+                },
+        }
+
+    def get(self):
+        queue = dqueue.core.Queue(request.args.get('queue', 'default'))
+
+        r = queue.expire_tasks()
+
+        logger.info("expired: %s", r)
+
+        return jsonify(tasks=r)
+
+
+app.add_url_rule(
+          '/tasks/expire',
+          view_func=ExpireTasks.as_view('expire_tasks'),
+          methods=['GET']
+)
+
 class ViewLogView(SwaggerView):
     operationId = "view"
 
