@@ -61,7 +61,7 @@ def log_info(queue):
     for q in queue.list_queues(None):
         logger.info(colored(q, 'green'))
         logger.info("; ".join(
-                f"{k}: {len(v)}" for k,v in q.info.items()
+                f"{k}: {v}" for k,v in q.summary.items()
             ))
         logger.info("\n")
 
@@ -92,9 +92,11 @@ def console_size():
 @click.option("-d", "--debug", default=False, is_flag=True)
 @click.option("-l", "--log", default=False, is_flag=True)
 @click.option("-i", "--info", default=False, is_flag=True)
+@click.option("-s", "--state", default=None)
+#@click.option("-a", "--max-age", default=None)
 @click.pass_obj
-def list(obj, debug, log, info):
-    for task in obj['queue'].list_tasks():
+def list(obj, debug, log, info, state):
+    for task in obj['queue'].list_tasks(state=state):
 
         td = task['task_dict']['task_data']
 
@@ -162,9 +164,12 @@ def view(obj, follow, since=0):
 
     info_cadence = 10
     till_next_info = info_cadence
+    last_info_time = 0
 
     task_info_cache={}
 
+    active_workers = {}
+
     while True:
         new_messages = obj['queue'].view_log(since=since)['event_log']
 
@@ -193,20 +198,22 @@ def view(obj, follow, since=0):
                 except Exception as e:
                     logger.error("very stange task: %s; %s", ti.keys(), e)
                     name = "missing"
-                    
+
             if name is None or name == "??": # ???
                 try:
                     m = json.loads(l['message'])
                     name = m['params']['node']
                     l['message'] = f"{m['qs']['job_id'][0][:8]} {m['params']['message']}"
                 except Exception as e:
-                    logger.error("%s", repr(e))
+                    logger.debug("%s", repr(e))
 
             print(("{since} {timestamp} "+colored("{task_key:10s}", "red") + " {message:40s} "+colored("{name:20s}", "yellow") + colored(" {worker_id:40s}", "cyan") ).format(
                     since=since,
                     name=name,
                     **l))
 
+            active_workers[l['worker_id']] = time.time()
+
             logger.debug(l)
             
 
@@ -226,13 +233,18 @@ def view(obj, follow, since=0):
             till_next_info -= 1
 
             if till_next_info <=0:
-                print()
-                for k,v in obj['queue'].info.items():
-                    print(k, ":", len(v), end="; ")
-                print()
-
                 till_next_info = info_cadence
 
+        if time.time() - last_info_time > 5:
+            print()
+            log_info(obj['queue'])
+            print("\033[34m", "; ".join(f"{k}: {v}" for k,v in obj['queue'].summary.items()), "\033[0m")
+
+            recent_workers = [k for k,v in active_workers.items() if v>time.time()-120]
+            print(f"\033[35m{len(recent_workers)} recent workers: {recent_workers}\033[0m")
+
+            last_info_time = time.time()
+
 
         time.sleep(1) # nobody ever needs anything but this default
 
@@ -292,11 +304,21 @@ def list_facts(obj):
 @click.pass_obj
 def guardian(obj, watch):
     while True:
+        #expre
+        print("exiure some tasks")
+        r = obj['queue'].expire_tasks()
+        print(colored("expired:", "yellow"), r)
+
         #try_all_locked
         print("trying to unlock something")
         task_data=obj['queue'].try_all_locked()
         print(colored("unlocked:", "green"), task_data)
         
+        #try_all_locked
+        print("trying to forgive failures")
+        task_data=obj['queue'].forgive_task_failures()
+        print(colored("forgiven:", "green"), task_data)
+        
         #clear event log 
         N = obj['queue'].clear_event_log(only_older_than_days=2./24.)
         print(f"cleared event log of {N} entries")
@@ -305,7 +327,7 @@ def guardian(obj, watch):
 
         print("getting queue statistics...")
         for k,v in obj['queue'].info.items():
-            print(k, ":", len(v), end="; ")
+            print(k, ":", v, end="; ")
         print("\n")
 
         print("sleeping", watch)
@@ -355,7 +377,7 @@ def start_executor(obj, deploy_runner_command, list_runners_command, timeout, ma
         r = obj['queue'].list_queues(None)
         for q in r:
             info = dict(q.info.items())
-            print(f"queue: \033[33m{q}\033[0m", "; ".join([ f"{k}: {len(v)}" for k, v in info.items() ]))
+            print(f"queue: \033[33m{q}\033[0m", "; ".join([ f"{k}: {v}" for k, v in info.items() ]))
             if len(info['waiting']) > 0:
                 print(f"\033[31mfound {len(info['waiting'])} waiting jobs, need to start some runners\033[0m")
                 
