@@ -16,7 +16,7 @@ from functools import reduce
 
 from bravado.client import SwaggerClient
 
-__version__ = "0.1.12"
+__version__ = "0.1.15"
 
 try:
     import io
@@ -422,17 +422,29 @@ class Queue:
     def get_worker_states(self):
         return [ model_to_dict(r) for r in EventLog.select().where(EventLog.worker_state!="unset").order_by(EventLog.timestamp.desc()).limit(1).execute(database=None) ]
 
-    def get(self):
+    def get(self, update_expected_in_s: float=-1):
         ""
         if self.current_task is not None:
             raise CurrentTaskUnfinished(self.current_task)
 
         self.note_worker_state("ready")
+
+        max_update_expected_in_s = 3600
+        default_update_expected_in_s = 1200
+
+        if update_expected_in_s > max_update_expected_in_s:
+            logger.warning("update expected timeout %s is too long, setting to maximum %s", update_expected_in_s, max_update_expected_in_s)
+            update_expected_in_s = max_update_expected_in_s
+
+        if update_expected_in_s <= 0:
+            logger.warning("no update expected timeout, setting to default %s", default_update_expected_in_s)
+            update_expected_in_s = default_update_expected_in_s
     
         r=TaskEntry.update({
                         TaskEntry.state:"running",
                         TaskEntry.worker_id:self.worker_id,
                         TaskEntry.modified:datetime.datetime.now(),
+                        TaskEntry.update_expected_in_s:update_expected_in_s
                     })\
                     .order_by(TaskEntry.created)\
                     .where( (TaskEntry.state=="waiting") & (TaskEntry.queue==self.queue) ).limit(1).execute(database=None)
@@ -785,12 +797,21 @@ class Queue:
         if len(entries) == 0:
             logger.info("no failed tasks: will not try to forgive")
             return 0
-            
-        logger.info("found %s failed tasks: will try to forgive", len(entries))
+
 
         entry=entries[0]
-        self.current_task=Task.from_task_dict(entry.task_dict_string)
-        self.current_task_stored_key=self.current_task.key
+
+        try:
+            self.current_task=Task.from_task_dict(entry.task_dict_string)
+            self.current_task_stored_key=self.current_task.key
+            logger.info("found %s failed tasks: will try to forgive", len(entries))
+        except Exception as e:
+            logger.info("will not forgive task %s with corrupt json, updating modified", entry.key)
+            r=TaskEntry.update({
+                        TaskEntry.modified:datetime.datetime.now(),
+                        TaskEntry.state: "corrupt",
+                    }).where(TaskEntry.key == entry.key).execute(database=None)
+            return 0
 
         task = self.current_task
 
@@ -839,14 +860,26 @@ class Queue:
             for key in self.list_tasks(fromk):
                 log("removing",fromk + "/" + key)
                 TaskEntry.delete().where(TaskEntry.key==key).execute(database=None)
+
+    @property
+    def summary(self):
+        r={}
+        for kind in "waiting","running","done","failed","locked":
+            r[kind] = TaskEntry.select().where(TaskEntry.state==kind, TaskEntry.queue==self.queue).count()
+           # .execute(database=None)
+            #
+        return r
         
     @property
     def info(self):
         ""
-        r={}
-        for kind in "waiting","running","done","failed","locked":
-            r[kind]=len(self.list_tasks(kind))
-        return r
+        return self.summary
+
+        #r = {}
+        #for kind in "waiting","running","done","failed","locked":
+        #    r[kind] = len(self.list_tasks(state=kind))
+
+        #return r
 
     def show(self):
         ""
@@ -958,4 +991,46 @@ class Queue:
 
         return jobs
 
+    def expire_tasks(self):
+        # yes. all of this can be one cmmand. but we want details
+        entries = TaskEntry.select().where(
+                    TaskEntry.state=="running",
+                ).order_by(TaskEntry.modified.desc()).limit(1).execute(database=None)
+
+        N = 0
+
+        for entry in entries:
+            age = datetime.datetime.now().timestamp() - entry.modified.timestamp()
+            expected = entry.update_expected_in_s
+
+            logger.info("running task with age %s limit %s", age, expected)
+            if age > expected:
+                logger.warning("to expire key %s state %s", entry.key, entry.state)
+
+                extra = {}
+
+                try:
+                    self.current_task=Task.from_task_dict(entry.task_dict_string)
+                    self.current_task_stored_key=self.current_task.key
+                    self.log_task("task failed - expired",self.current_task,"failed")
+                except Exception as e:
+                    logger.error("unexpected error in decoding task content: %s", repr(e))
+                    extra = {TaskEntry.task_dict_string: json.dumps({'corrupt_json': entry.task_dict_string})}
+                    self.log_task("task failed - corrupt json", None, "failed", task_key=entry.key)
+
+
+                n = TaskEntry.update({
+                            TaskEntry.state:"failed",
+                            **extra
+                        }).where(
+                            TaskEntry.state=="running",
+                            TaskEntry.key==entry.key,
+                        ).execute(database=None)
+
+                logger.warning("expired %s", n)
+
+                N += n
+
+        return N
+
 
