@@ -126,13 +126,13 @@ class QueueProxy(DataFacts, Queue):
                 ).response().result
 
 
-    def get(self):
+    def get(self, update_expected_in_s=-1):
         if self.current_task is not None:
             raise CurrentTaskUnfinished(self.current_task)
 
         print(dir(self.client.worker))
 
-        r = self.client.worker.getOffer(worker_id=self.worker_id, queue=self.queue).response()
+        r = self.client.worker.getOffer(worker_id=self.worker_id, queue=self.queue, update_expected_in_s= update_expected_in_s).response()
 
         if r.result is None:
             raise Empty()
@@ -208,8 +208,8 @@ class QueueProxy(DataFacts, Queue):
         self.logger.info("deleted %s", nentries)
 
 
-    def list_tasks(self):
-        l = [task for task in self.client.tasks.listTasks().response().result['tasks']]
+    def list_tasks(self, state="any"):
+        l = [task for task in self.client.tasks.listTasks(state=state).response().result['tasks']]
         self.logger.info(f"found tasks: {len(l)}")
         return l
 
@@ -220,6 +220,10 @@ class QueueProxy(DataFacts, Queue):
         for kind in "waiting","running","done","failed","locked":
             r[kind]=[t for t in tasks if t['state'] == kind]
         return r
+    
+    @property
+    def summary(self):
+        return self.client.tasks.summary(queue=self.queue).response().result['tasks']
 
     def show(self):
         r=""
@@ -233,6 +237,9 @@ class QueueProxy(DataFacts, Queue):
 
     def forgive_task_failures(self):
         return self.client.tasks.forgive_failures(worker_id=self.worker_id, queue=self.queue).response().result
+    
+    def expire_tasks(self):
+        return self.client.tasks.expire().response().result
 
     def callback(self, url, params):
         """
